import React from 'react'
import styled from 'styled-components'
import { Action, Link, Icon } from '@pubsweet/ui'
import { th } from '@pubsweet/ui-toolkit'
import { HTMLString } from './ui'
import Mailer from './mailer'
import { States } from './dashboard'

const ManuscriptHead = styled.div`
  display: flex;
  flex-direction: row;
  justify-content: center;
  align-items: stretch;
  background-color: ${th('colorTextReverse')};
  border-bottom: ${th('borderWidth')} ${th('borderStyle')} ${th('colorBorder')};
  font-weight: 600;
  max-height: calc(${th('gridUnit')} * 8);
  & > div {
    margin-bottom: 0;
    height: 100%;
    overflow: auto;
    & > div {
      padding: calc(${th('gridUnit')} * 2.5) calc(${th('gridUnit')} * 2);
    }
  }
`
const LeftSide = styled.div`
  flex: 1 1 1000px;
  & > div {
    max-width: 1000px;
    margin: 0 0 0 auto;
    white-space: nowrap;
    overflow: hidden;
    text-overflow: ellipsis;
  }
`
const RightSide = styled.div`
  flex: 1 1 500px;
  z-index: 1;
  & > div {
    max-width: 500px;
    margin: 0 auto 0 0;
    text-align: right;

    a,
    button {
      display: inline-flex;
      align-items: center;
    }
  }
`
class MailLink extends React.Component {
  state = { show: false }
  render() {
    return (
      <React.Fragment>
        <Action onClick={() => this.setState({ show: true })}>
          <Icon color="currentColor" size={2.5}>
            mail
          </Icon>
          Helpdesk
        </Action>
        {this.state.show && (
          <Mailer
            close={() => this.setState({ show: false })}
            currentUser={this.props.currentUser}
            manuscript={this.props.manuscript}
            recipients={['helpdesk']}
          />
        )}
      </React.Fragment>
    )
  }
}

const SubmissionHeader = BaseComponent => ({
  currentUser,
  manuscript,
  children,
  ...props
}) => (
  <React.Fragment>
    <ManuscriptHead>
      <LeftSide>
        <div>
          {manuscript.id}: <HTMLString string={manuscript.meta.title} />
        </div>
      </LeftSide>
      <RightSide>
        <div>
          {currentUser.admin ? (
            <React.Fragment>
              {window.location.pathname ===
              `/submission/${manuscript.id}/activity` ? (
                <React.Fragment>
                  {States.admin[manuscript.status].url !== 'activity' && (
                    <Link
                      to={`/submission/${manuscript.id}/${
                        States.admin[manuscript.status].url
                      }`}
                    >
                      <Icon color="currentColor" size={2.5}>
                        x-square
                      </Icon>
                      Exit activity
                    </Link>
                  )}
                </React.Fragment>
              ) : (
                <Link to={`/submission/${manuscript.id}/activity`}>
                  <Icon color="currentColor" size={2.5}>
                    check-square
                  </Icon>
                  Activity
                </Link>
              )}
            </React.Fragment>
          ) : (
            <MailLink currentUser={currentUser} manuscript={manuscript} />
          )}
        </div>
      </RightSide>
    </ManuscriptHead>
    <BaseComponent currentUser={currentUser} manuscript={manuscript} {...props}>
      {children}
    </BaseComponent>
  </React.Fragment>
)

export default SubmissionHeader
