import { states } from 'config'
import React from 'react'
import { Button, H1, H2, H3 } from '@pubsweet/ui'
import { B, Buttons, Page } from '../ui'
import { FileThumbnails } from '../preview-files'
import Citation from './Citation'

const SubmitComplete = ({ manuscript, history }) => {
  const { status, meta, files, journal } = manuscript
  const { fundingGroup, releaseDelay } = meta
  const curr = states.indexOf(status)
  const done = states.indexOf('xml-complete')
  return (
    <Page>
      <H1>Thank you for your submission</H1>
      <p>
        {`Your submission is `}
        {curr >= done
          ? 'complete and accepted for archiving. If you have any questions, or require changes to be made, please click the link above to contact the Helpdesk.'
          : `${
              ['submitted', 'in-review', 'xml-review'].includes(status)
                ? 'in review'
                : 'being processed'
            }. You will receive email updates as it is processed. You can also log in to Europe PMC plus at any time to check the status of your submission. If you require any changes to the following information, please click the link above to contact the Helpdesk.`}
      </p>
      <Buttons right>
        <Button onClick={() => history.push('/')} primary>
          Done
        </Button>
      </Buttons>
      <H2>Submission details</H2>
      <H3>Citation</H3>
      <Citation journal={journal} metadata={meta} />
      <H3>Files</H3>
      <FileThumbnails files={files} />
      <H3>Funding</H3>
      {fundingGroup && fundingGroup.length > 0 && (
        <p>
          <B>Grants: </B>
          {fundingGroup.map((f, t) => (
            <span key={f.awardId}>
              {`${f.fundingSource} ${f.awardId}`}
              {t !== fundingGroup.length - 1 && ', '}
            </span>
          ))}
        </p>
      )}
      {(releaseDelay || typeof releaseDelay === 'number') && (
        <p>
          <B>Embargo: </B>
          {`${releaseDelay} month${
            parseInt(releaseDelay, 10) === 1 ? '' : 's'
          }`}
        </p>
      )}
    </Page>
  )
}

export default SubmitComplete
